(function () {
    'use strict';

    var babelHelpers = {};
    babelHelpers.typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
      return typeof obj;
    } : function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj;
    };

    babelHelpers.extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    babelHelpers;

    /* In regards to performance, I worry about excessive iteration, which is why there is
     * a less functional and more imperative style.
     */
    var exportedThings = undefined;
    var wordSoFar = undefined;
    var startDelete = undefined;
    var endDelete = undefined;
    var isDefault = undefined;
    var waitingFor = undefined;
    var thenAfterThat = undefined;
    var whitespaceOptional = undefined;
    var foundWhitespace = undefined;
    var identifierEndingChars = /[\s\(;=]/;
    var endOfStatementRegex = /[\n}\);]/;

    function init() {
        reset('', -1);
        exportedThings = [];
        waitingFor = 'EXPORT_KEYWORD';
    }

    function getParse() {
        return exportedThings;
    }

    function newCharacter(char, index) {
        wordSoFar += char;
        // console.log(`"${wordSoFar}" -- ${waitingFor} -- ${char} -- ${index} -- ${startDelete}`)

        if (waitingFor === 'EXPORT_KEYWORD') {
            if (!'export'.startsWith(wordSoFar)) {
                // console.log(`"${wordSoFar}" isn't the export keyword`)
                reset(char, index);
            } else if ('export' === wordSoFar) {
                // console.log('\nfound export keyword')
                waitingFor = 'WHITESPACE';
                thenAfterThat = 'DEFAULT';
                wordSoFar = '';
            }
        } else if (waitingFor === 'DEFAULT') {
            if (!'default'.startsWith(wordSoFar)) {
                // console.log(`"${wordSoFar}" isn't the default keyword`)
                waitingFor = 'TYPE';
            } else if ('default' === wordSoFar) {
                // console.log(`Found default keyword`)
                isDefault = true;
                waitingFor = 'WHITESPACE';
                thenAfterThat = 'TYPE';
                wordSoFar = '';
            }
        } else if (waitingFor === 'TYPE') {
            if ('function'.startsWith(wordSoFar)) {
                if ('function' === wordSoFar) {
                    if (isDefault) {
                        // we don't need the name if it's default, because the name of the export is "default"
                        addNewExport(char, index, {
                            startDelete: startDelete,
                            endDelete: index - 'function'.length,
                            isDefault: isDefault
                        });
                    } else {
                        waitingFor = 'IDENTIFIER';
                        endDelete = index - wordSoFar.length + 1;
                        wordSoFar = '';
                    }
                }
            } else if ('var'.startsWith(wordSoFar)) {
                if ('var' === wordSoFar) {
                    waitingFor = 'IDENTIFIER';
                    endDelete = index - wordSoFar.length + 1;
                    wordSoFar = '';
                }
            } else if ('const'.startsWith(wordSoFar)) {
                if ('const' === wordSoFar) {
                    waitingFor = 'IDENTIFIER';
                    endDelete = index - wordSoFar.length + 1;
                    wordSoFar = '';
                }
            } else if ('let'.startsWith(wordSoFar)) {
                if ('let' === wordSoFar) {
                    waitingFor = 'IDENTIFIER';
                    endDelete = index - wordSoFar.length + 1;
                    wordSoFar = '';
                }
            } else if (isDefault) {
                // this is a default exported expression
                addNewExport(char, index, {
                    startDelete: startDelete,
                    endDelete: index - wordSoFar.length,
                    isDefault: isDefault,
                    name: null
                });
            } else {
                return {
                    msg: 'Non default exports must have a name associated with them',
                    startDelete: startDelete,
                    endDelete: index
                };
            }
        } else if (waitingFor === 'IDENTIFIER') {
            if (/^\s$/.test(wordSoFar)) {
                wordSoFar = '';
            } else if (identifierEndingChars.test(char)) {
                addNewExport(char, index, {
                    startDelete: startDelete,
                    endDelete: endDelete,
                    isDefault: isDefault,
                    name: wordSoFar.substring(0, wordSoFar.length - 1)
                });
            }
        } else if (waitingFor === 'WHITESPACE') {
            wordSoFar = wordSoFar.replace(endOfStatementRegex, '');
            if (/\s/.test(char)) {
                foundWhitespace = true;
            } else if (foundWhitespace) {
                // console.log(`"${char}" is not whitespace`)
                // console.log(`switching from whitespace to ${thenAfterThat}`);
                if (thenAfterThat === 'EXPORT_KEYWORD') {
                    startDelete = index - wordSoFar.trim().length + 1;
                }
                waitingFor = thenAfterThat;
                thenAfterThat = null;
                foundWhitespace = false;
                wordSoFar = wordSoFar.trim();
            } else {
                if (whitespaceOptional) {
                    if (thenAfterThat === 'EXPORT_KEYWORD') {
                        startDelete = index - wordSoFar.trim().length + 1;
                    }
                    waitingFor = thenAfterThat;
                    thenAfterThat = null;
                    foundWhitespace = false;
                    wordSoFar = wordSoFar.trim();
                } else {
                    reset(char, index);
                }
            }
        } else if (waitingFor === 'END_OF_STATEMENT') {
            if (endOfStatementRegex.test(wordSoFar)) {
                waitingFor = 'WHITESPACE';
                thenAfterThat = 'EXPORT_KEYWORD';
                wordSoFar = char;
                whitespaceOptional = true;
            } else {
                // console.log(`not end of statement = "${wordSoFar}"`)
            }
        } else {
                throw new Error('Unknown export parse state -- waiting for "' + waitingFor + '"');
            }
    }

    function reset(char, index) {
        waitingFor = 'END_OF_STATEMENT';
        whitespaceOptional = true;
        wordSoFar = char;
        startDelete = index + 1;
        isDefault = false;
        foundWhitespace = false;
        whitespaceOptional = false;
    }

    function addNewExport(char, index, opts) {
        // console.dir(opts);
        exportedThings.push(opts);
        reset(char, index);
    }

    /* In terms of performance, I'm not as worried about excessive iteration here, but rather string
     * manipulation operations. Hence a more functional style.
     */

    var nameOfDefaultExportVar = '$__default__export';

    function toSystemRegister(source, exportedThings, exportUpdates, importedThings) {
        var compiled = source;

        var mutationsToPerform = exportedThings.map(function (exportedThing) {
            return babelHelpers.extends({}, exportedThing, { type: 'export_statement' });
        }).concat(exportUpdates.map(function (exportUpdate) {
            return babelHelpers.extends({}, exportUpdate, { type: 'export_update' });
        })).concat(importedThings.map(function (importedThing) {
            return babelHelpers.extends({}, importedThing, { type: 'import' });
        })).sort(function (first, second) {
            // we want the indices to be from biggest to smallest so that mutations don't interfere with each other.
            return getRelevantIndex(second) - getRelevantIndex(first);
        });

        // console.dir(mutationsToPerform)

        mutationsToPerform.forEach(function (mutation) {
            if (mutation.type === 'export_statement') {
                compiled = applyExportStatementMutation(compiled, mutation);
            } else if (mutation.type === 'export_update') {
                compiled = applyExportUpdateMutation(compiled, mutation);
            } else if (mutation.type === 'import') {
                compiled = applyImportMutation(compiled, mutation);
            } else {
                throw new Error('Unknown mutation type \'' + mutation.type + '\'');
            }
        });

        // console.dir('--------------------------------------------------------')

        var result = 'System.register([' + importedModuleIdentifiers(importedThings) + '], function($__export) {' + importedVariables(importedThings) + '\n    return {\n        setters: [' + importedThings.map(function (importedThing) {
            return importSetter(importedThing);
        }) + '\n        ],\n        execute: function() {\n            ' + compiled.replace(/\n/g, '\n            ') + '\n        }\n    };\n});';

        return result;
    }

    function applyExportUpdateMutation(string, exportUpdate) {
        var exportedValue = undefined;
        if (exportUpdate.value) {
            exportedValue = exportUpdate.value;
        } else if (exportUpdate.name === 'default') {
            exportedValue = nameOfDefaultExportVar;
        } else {
            exportedValue = exportUpdate.name;
        }
        var mutation = ';$__export(\'' + exportUpdate.name + '\', ' + exportedValue + ');';
        return string.substring(0, exportUpdate.insertionIndex + 1) + mutation + string.substring(exportUpdate.insertionIndex + 1);
    }

    function applyExportStatementMutation(string, exportedThing) {
        var result = undefined;

        if (exportedThing.isDefault && !exportedThing.name) {
            result = string.substring(0, exportedThing.startDelete) + ('var ' + nameOfDefaultExportVar + ' = ') + string.substring(exportedThing.endDelete);
        } else {
            result = string.substring(0, exportedThing.startDelete) + string.substring(exportedThing.endDelete);
        }

        return result;
    }

    function applyImportMutation(string, importedThing) {
        return string.substring(0, importedThing.startIndex) + string.substring(importedThing.endIndex + 1);
    }

    function getRelevantIndex(mutation) {
        if (mutation.type === 'export_statement') {
            return mutation.endDelete;
        } else if (mutation.type === 'export_update') {
            return mutation.insertionIndex;
        } else if (mutation.type === 'import') {
            return mutation.endIndex;
        } else {
            throw new Error('Unknown mutation type \'' + mutation.type + '\'');
        }
    }

    function importedModuleIdentifiers(importedThings) {
        return importedThings.map(function (importedThing) {
            return "'" + importedThing.moduleIdentifier + "'";
        }).join(', ');
    }

    function importedVariables(importedThings) {
        if (importedThings.length === 0) {
            return '';
        } else {
            return '\n    var ' + importedThings.map(function (importedThing) {
                return (importedThing.defaultImportName ? importedThing.defaultImportName : '') + importedThing.namedImports.map(function (namedImport) {
                    return namedImport.alias ? namedImport.alias : namedImport.name;
                });
            }).join(', ') + ';';
        }
    }

    function importSetter(importedThing) {
        function defaultImportAssignment(importedThing) {
            return importedThing.defaultImportName ? '\n                ' + importedThing.defaultImportName + ' = $__mod.default;' : '';
        }

        function namedImportAssignment(namedImport) {
            return '\n                ' + (namedImport.alias ? namedImport.alias : namedImport.name) + ' = $__mod.' + namedImport.name;
        }

        return '\n            function($__mod) {' + defaultImportAssignment(importedThing) + importedThing.namedImports.map(function (namedImport) {
            return namedImportAssignment(namedImport);
        }) + '\n            },';
    }

    /* Shadowed variables are our enemy
     */

    var parenLevel = undefined;
    var bracesLevel = undefined;
    var currentWord = undefined;
    var thingsToExport = undefined;
    var exportChanges = undefined;
    var hasDefaultExport = undefined;
    var endOfDefaultExport = undefined;
    var addedDefaultExportMutation = undefined;
    var nameOfDefaultExport = undefined;
    function init$1(exportsParse) {
        currentWord = '';
        resetWord([], null, -1);
        exportChanges = [];
        thingsToExport = [];
        endOfStatement([], null, -1);
        parenLevel = 0;
        bracesLevel = 0;

        addedDefaultExportMutation = false;
        var defaultExports = exportsParse.filter(function (_export) {
            return _export.isDefault;
        });
        if (defaultExports.length === 1) {
            hasDefaultExport = true;
            endOfDefaultExport = defaultExports[0].endDelete;
            nameOfDefaultExport = defaultExports[0].name;
        } else {
            hasDefaultExport = false;
            endOfDefaultExport = null;
        }
    }

    function getUpdates() {
        return exportChanges;
    }

    /* In regards to performance, I worry about excessive iteration, which is why there is
     * a less functional and more imperative style.
     */
    function newCharacter$1(exportsParse, char, index) {
        currentWord += char;

        switch (char) {
            case '(':
                parenLevel++;
                resetWord(exportsParse, char, index);
                break;

            case ')':
                parenLevel--;
                resetWord(exportsParse, char, index);
                break;

            case ';':
                // what we really want from this if statement is if we're not inside of an expression or object literal
                if (bracesLevel === 0 && parenLevel === 0) {
                    endOfStatement(exportsParse, char, index);
                } else {
                    resetWord(exportsParse, char, index);
                }
                break;

            case '{':
                bracesLevel++;
                resetWord(exportsParse, char, index);
                break;
            case '}':
                bracesLevel--;
                resetWord(exportsParse, char, index);
                break;

            case '\n':
            case '\r':
                // what we really want from this if statement is if we're not inside of an expression or object literal
                if (bracesLevel === 0 && parenLevel === 0) {
                    endOfStatement(exportsParse, char, index);
                } else {
                    resetWord(exportsParse, char, index);
                }
                break;

            // the things that just reset the word
            case "'":
            case '=':
            case ':':
            case '-':
            case '+':
            case '/':
            case '^':
                resetWord(exportsParse, char, index);
                break;

            default:
                if (/\s/.test(char)) {
                    resetWord(exportsParse, char, index);
                } else if (/[a-zA-z0-9]+/.test(char)) {
                    // just let the current word accrue more letters
                } else {
                        // console.log(`Unknown char ${char}`);
                    }
                break;
        }
    }

    function resetWord(parse, char, index) {
        checkExportedThingsChanged(parse);

        // console.log(`resetting word -- '${currentWord}'`)
        currentWord = '';
    }

    function checkExportedThingsChanged(parse) {
        /* The current character is the one that caused this function to be called, so it should not count
         * as part of the current word
        */
        var relevantWord = currentWord.substring(0, currentWord.length - 1);
        // console.log(`check things changed -- '${relevantWord}'`);

        for (var i = 0; i < parse.length; i++) {
            var nameOfExp = parse[i].name;
            if (nameOfExp === relevantWord) {
                thingsToExport.push(relevantWord);
            }
        }
    }

    function endOfStatement(parse, char, index) {
        resetWord(parse, char, index);

        if (char === '\n' || char === '\r') {
            //we want the export call on the same line as the mutation of the value (source mapping???)
            index--;
        }

        if (parenLevel === 0 && bracesLevel === 0 && hasDefaultExport && !addedDefaultExportMutation && index > endOfDefaultExport && !nameOfDefaultExport) {
            // console.log(`EXPORTING_____________ 'default'`)
            addedDefaultExportMutation = true;
            exportChanges.push({
                name: 'default',
                value: null, /* deduced by system-register-writer */
                insertionIndex: index
            });
        }
        for (var i = 0; i < thingsToExport.length; i++) {
            // console.log(`EXPORTING named export '${thingsToExport[i]}'`)
            var name = thingsToExport[i];
            if (hasDefaultExport && !addedDefaultExportMutation && index > endOfDefaultExport && nameOfDefaultExport === thingsToExport[i]) {
                addedDefaultExportMutation = true;
                name = 'default';
            }
            exportChanges.push({
                name: name,
                value: thingsToExport[i],
                insertionIndex: index
            });
        }

        thingsToExport = [];
    }

    var bracesLevel$1 = undefined;
    var parensLevel = undefined;
    var currentWord$1 = undefined;
    var startIndex = undefined;
    var endIndex = undefined;
    var defaultImportName = undefined;
    var namedImports = undefined;
    var moduleIdentifier = undefined;
    var moduleIdentifierDelimiter = undefined;
    var imports = undefined;
    var lookingFor = undefined;
    function init$2() {
        bracesLevel$1 = 0;
        parensLevel = 0;
        imports = [];
        resetWord$1();
        resetImport();
    }

    function getImports() {
        return imports;
    }

    function newCharacter$2(char, index) {
        currentWord$1 += char;

        if (lookingFor === 'IMPORT_STATEMENT') {
            if (char === '{' && lookingFor !== 'IMPORTED_VARS') {
                bracesLevel$1++;
                resetWord$1();
            } else if (char === '}') {
                bracesLevel$1--;
                resetWord$1();
            } else if (char === '(') {
                parensLevel++;
                resetWord$1();
            } else if (lookingFor === 'IMPORT_STATEMENT' && currentWord$1 === 'import') {
                startIndex = index - 'import'.length;
                // console.log('looking for imported vars');
                lookingFor = 'IMPORTED_VARS';
                resetWord$1();
            }
        } else if (lookingFor === 'IMPORTED_VARS') {
            if (/\s/.test(char)) {
                resetWord$1();
            } else if (char === '{') {
                // console.log('looking for variable list');
                namedImports = [];
                lookingFor = 'VARIABLE_LIST';
                resetWord$1();
            } else {
                // console.log('looking for default import var name');
                resetWord$1();
                currentWord$1 = char;
                lookingFor = 'DEFAULT_IMPORT_VAR_NAME';
            }
        } else if (/\s/.test(char) && lookingFor === 'DEFAULT_IMPORT_VAR_NAME') {
            // console.log('found default import name ' + currentWord);
            defaultImportName = currentWord$1.substring(0, currentWord$1.length - 1);
            resetWord$1();
            if (namedImports === null) {
                // console.log('looking for named imports')
                lookingFor = 'NAMED_IMPORTS';
            } else {
                // console.log('looking for ')
                lookingFor = 'FROM_KEYWORD';
            }
        } else if (lookingFor === 'NAMED_IMPORTS') {
            if (char === '{') {
                nameImports = [];
                resetWord$1();
                lookingFor = 'VARIABLE_LIST';
            } else if (/\s/.test(char)) {
                resetWord$1();
            } else {
                lookingFor = 'FROM_KEYWORD';
            }
        } else if (lookingFor === 'VARIABLE_LIST') {
            if (/\s/.test(char)) {
                if (currentWord$1.length > 1) {
                    // console.log(`looking for named imports comma, current word == ${currentWord}`);
                    namedImports.push({
                        name: currentWord$1.trim()
                    });
                    lookingFor = 'NAMED_IMPORTS_COMMA';
                }
                resetWord$1();
            } else if (char === '}') {
                // console.log('looking for from keyword');
                if (currentWord$1.length > 1) {
                    namedImports.push({
                        name: currentWord$1.substring(0, currentWord$1 - 1)
                    });
                }
                resetWord$1();
                lookingFor = 'FROM_KEYWORD';
            } else if (char === ',') {
                // console.log('looking for named imports');
                if (currentWord$1.length > 1) {
                    namedImports.push({
                        name: currentWord$1.substring(0, currentWord$1.length - 1)
                    });
                }
                resetWord$1();
                lookingFor = 'VARIABLE_LIST';
            }
        } else if (lookingFor === 'NAMED_IMPORTS_COMMA') {
            if (char === ',') {
                // console.log('looking for named imports')
                lookingFor = 'NAMED_IMPORTS';
                resetWord$1();
            } else if (/\s/.test(char)) {
                resetWord$1();
            } else if (char === '}') {
                // console.log('looking for from keyword');
                if (currentWord$1.length > 1) {
                    namedImports.push({
                        name: currentWord$1.substring(0, currentWord$1.length - 1)
                    });
                }
                lookingFor = 'FROM_KEYWORD';
                resetWord$1();
            } else {
                throw new Error('Unexpected character while looking for NAMED_IMPORTS_COMMA -- \'' + char + '\'');
            }
        } else if (lookingFor === 'FROM_KEYWORD') {
            if (currentWord$1 === 'from') {
                // console.log('looking for string to begin module identifier')
                lookingFor = 'MODULE_IDENTIFIER_BEGINNING';
                resetWord$1();
            } else if (/\s/.test(char)) {
                resetWord$1();
            }
        } else if (lookingFor === 'MODULE_IDENTIFIER_BEGINNING') {
            if (/\s/.test(char)) {
                resetWord$1();
            } else if (char === "'") {
                // console.log('looking for module identifier')
                moduleIdentifierDelimiter = "'";
                lookingFor = 'MODULE_IDENTIFIER';
                resetWord$1();
            } else if (char === '"') {
                // console.log('looking for module identifier')
                moduleIdentifierDelimiter = '"';
                lookingFor = 'MODULE_IDENTIFIER';
                resetWord$1();
            }
        } else if (char === moduleIdentifierDelimiter) {
            lookingFor = 'END_OF_STATEMENT';
            moduleIdentifier = currentWord$1.substring(0, currentWord$1.length - 1);
            // console.log('looking for end of statement')
        } else if (lookingFor === 'END_OF_STATEMENT') {
                if (!(char === ';' || /\s/.test(char))) {
                    //we have reached the end of the import statement
                    imports.push({
                        moduleIdentifier: moduleIdentifier,
                        defaultImportName: defaultImportName,
                        namedImports: namedImports || [],
                        startIndex: startIndex,
                        endIndex: index - 1
                    });
                    // console.log(imports[imports.length - 1]);
                    resetImport();
                }
            }
    }

    function resetWord$1() {
        // console.log(currentWord);
        currentWord$1 = '';
    }

    function resetImport() {
        resetWord$1();
        startIndex = -1;
        endIndex = -1;
        defaultImportName = null;
        namedImports = null;
        lookingFor = 'IMPORT_STATEMENT';
        moduleIdentifierDelimiter = '';
        moduleIdentifier = '';
    }

    /* In regards to performance, I worry about excessive iteration, which is why there is
     * a less functional and more imperative style.
     */
    function compile(string) {
        if (typeof string !== 'string') {
            throw new Error('Cannot compile a non-string \'' + (typeof string === 'undefined' ? 'undefined' : babelHelpers.typeof(string)) + '\' -- \'' + string + '\'');
        }
        // console.log(string + '\n------------------------------------------\n')
        init();
        init$2();
        var currentLine = 1;
        var currentCharacterNumber = 1;
        for (var i = 0; i < string.length; i++) {
            if (string[i] === '\n' || string[i] === '\r') {
                currentLine++;
                currentCharacterNumber = 1;
            } else {
                currentCharacterNumber++;
            }
            var errExports = newCharacter(string[i], i);
            if (errExports) {
                throw new Error('On line ' + currentLine + ', character ' + currentCharacterNumber + ': ' + errExports.msg + ' -- "' + string.substring(errExports.startIndex, errExports.endIndex).trim() + '"');
            }

            var errImports = newCharacter$2(string[i], i);
            if (errImports) {
                throw new Error('On line ' + currentLine + ', character ' + currentCharacterNumber + ': ' + errImports.msg + ' -- "' + string.substring(errImports.startIndex, errImports.endIndex).trim() + '"');
            }
        }

        // console.log('\n----------------------------------------\n');

        var exportsParse = getParse();

        currentLine = 1;
        currentCharacterNumber = 1;
        init$1(exportsParse);
        for (var i = 0; i < string.length; i++) {
            if (string[i] === '\n' || string[i] === '\r') {
                currentLine++;
                currentCharacterNumber = 1;
            } else {
                currentCharacterNumber++;
            }
            var err = newCharacter$1(exportsParse, string[i], i);
            if (err) {
                throw new Error('On line ' + currentLine + ', character ' + currentCharacterNumber + ': ' + err.msg + ' -- "' + string.substring(err.startIndex, err.endIndex).trim() + '"');
            }
        }

        // console.log('\n----------------------------------------\n');

        var exportUpdates = getUpdates();
        var importedThings = getImports();

        return toSystemRegister(string, exportsParse, exportUpdates, importedThings);
    }

    if (exports) {
        // node
        exports.compile = compile;
    } else if (typeof window !== 'undefined') {
        // browser
        window.kremling = compile;
    } else if (typeof self !== 'undefined') {
        self.kremling = compile;
    }

}());